#!/usr/bin/perl

#######################################################################
# Serial Print of Experiment Sheets for Production Experiments
#
# author: Gerhard Kremer, 2010
#
# usage:
# ./print-exp.perl [options]
# 
# purpose:
# Script creates handouts for participants
# (instructions, stimuli sheets, questionnaire)
# and prints one stimulus per sheet;
# creates file for annotation purposes and a logfile
# for each language group (DE, IT)
#######################################################################


use warnings;
no warnings qw(syntax);

use Getopt::Std;
getopts('l:s:n:h');
use vars qw($opt_l $opt_s $opt_n $opt_h
	  $wday $isdst $nused $yday);

# print help message
if ($opt_h)
  {die "\nThis script randomises stimuli
and prints handouts for experiment participants
involved in our feature production experiment\n
Usage:
./print-exp.pl [-h] [-l (en|it)] [-s subjectno] [-n addnsubjs]
-l: language (def. de)
-s: start number of subject
-n: add this number of subjects \n\n"};

# init:
# set language, first and last subj no
$lang = ($opt_l ? $opt_l : die "Choose a language\n"); # de/it
if ($lang !~ /^(de|it)$/){die "language abbrev. is wrong...\n"};
$startsubjno = ($opt_s ? $opt_s : 1);
$lastsubjno = ($opt_n ? $startsubjno + $opt_n -1 : $startsubjno +1);
$nowsubjno = $startsubjno;

# set file names
$instrfile = "instructions".uc($lang).".tex";
$instrpsfile = "instructions".uc($lang).".ps";
$annotfile = "prod-annot".uc($lang).".txt";
$logfile  = "prod-log".uc($lang).".txt";
$stimulifile = "StimuliProdEx.txt";
$stimtexfile = "stimiprint.tex";
$stimipsfile = "stimiprint.ps";
$quesorigfile = "questionnaire".uc($lang).".tex";
$questmpfile = "questmp.tex";
$questmppsfile = "questmp.ps";

# read experiment logfile 
# get subj numbers of already printed sheets
$maxsubjno = 0;
open(EXPLOGIN, "< $logfile")
  || die "Could not find Logfile $logfile --\n only in case of 1st use create an empty file with that name\n";
while(<EXPLOGIN>){
  if($_ =~ /^\# s(\d+)p(\d+)/){
    $nused{subjno}{$1}=1; 
    $maxsubjno=$1 unless $maxsubjno > $1;
  };
}
close EXPLOGIN;

# check if subj numbers were already used
# i.e. if start no is lower than highest no logged
if ( $startsubjno <= $maxsubjno )
  {die "So far, the highest subj number used: $maxsubjno\n",
       "Choose a different start number.\n"};

# user feedback 
print "I use language  - $lang -\n",
  "and start with subj no.: $startsubjno\n";


# execution block for 1 stimuli set ^= 2 participants
SUBJECT:{
#
# read stimuli list
# in randomised order
# (defined by array of English words)

sleep(1); 
# mawk sets seed for randomisation from clock (same seed within 1 s)

@rndord = (); # delete previous random stimuli set
open( RNDORD,
      q[mawk '{print rand()";" $0}' ].$stimulifile.q[ | sort -n | cut -d ";" -f2 |] )
  or die "Couldn't randomise\n";

while (<RNDORD>){
  ($cat, $en, $de, $it) = split(' ', $_);
  $stimi{$en}{cat} = $cat;
  $stimi{$en}{de} = $de;
  $stimi{$en}{it} = $it;
#  $stimi{$en}{number} = $.; #print $.;
  push (@rndord, $en);
};
close RNDORD;

# create stimuli tex file for each stimulus, 
# compile, print,
# write logfile
foreach $range ("0..24", "25..49"){# 2 sets of 25 stimuli (50 in total)
  last if $nowsubjno > $lastsubjno;

# tex file preamble
  open (TEX , "> $stimtexfile") or die "Couldn't open $stimtexfile\n";
  ($tex = <<'ENDoPREAMBLE' ) =~ s/^\s+//gm;
    \documentclass[a4paper,DIV14]{scrartcl}
    \usepackage[latin1]{inputenc}
    \usepackage[absolute]{textpos}
    \pagestyle{empty}
    \begin{document}
ENDoPREAMBLE

# tex file body w stimulus inserted and end of file
  $pno = 0; # init participant no processed
  @ra = eval $range;
  foreach $enstim (@rndord[@ra]) {# each participant
    $pno++;
    $stlabel = "s${nowsubjno}p${pno}"; #label sxxxpxx (subj, page)
    $stimulus = $stimi{$enstim}{$lang};
      ($tex .= 
       <<'STIMULUS'."$stimulus\n".<<'LABEL'."$stlabel\n".<<'EOD') =~ s/^\s*//gm;
	\begin{center}
	\huge
	\textbf{\textsf{%
STIMULUS
	}}
	\end{center}
	\begin{textblock}{2}[0,0](0.7,15.3)
	\ttfamily
LABEL
	\end{textblock}
	\newpage
EOD
# progress: print '.';
    $explogout .= "# $stlabel $enstim $stimulus\n";
  };# foreach english stimulus word
  $tex .= '\end{document}';#,"\n";
  print TEX $tex;
  close TEX;

# generate questionnaire tex file (w/ subjno label)
open(QUE,"< $quesorigfile")
  || die "Could not open questionnaire file\n";
$quetex="";
while(<QUE>){
  $quetex .= $_;
  last if /\Q\begin{document}/;
};
$sblabel = "s${nowsubjno}\n";
($quetex .= <<'LABEL'."$sblabel".<<'EOD' ) =~ s/^\s*//gm;
	\begin{textblock}{2}[0,0](0.7,15.3)
	\texttt{%
LABEL
        }
	\end{textblock}
EOD
while(<QUE>){$quetex .= $_};
close QUE;

open(QUETEX,"> $questmpfile") or die "Couldn't open $questmpfile\n";
print QUETEX $quetex;
close QUETEX;

# tex/regenerate ps files from tex sources (if necessary)
system("rubber -q --ps $instrfile $stimtexfile $questmpfile") == 0 
    or system("latex $instrfile; latex $stimtexfile; latex $questmpfile") == 0 
    or die "Could not compile PS files from TEX files - chack script and system\n";

# print sheets
system("lpr -P HP4350 $instrpsfile $stimipsfile $questmppsfile") == 0 or die "Could not print PS files \n check lpr installation on system?";

  print "Sent subject $nowsubjno to ... printer\n";

# stop block repeat loop if we have reached desired participant number
  last SUBJECT if $nowsubjno >= $lastsubjno;
  $nowsubjno++;
};# foreach subset of stimuli from the whole stimuli set
redo SUBJECT;
}; # SUBJECT block

# get time and
# write log and annotation files
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
  localtime(time);
$year = sprintf("%02d", $year % 100); #+1900

open(EXPLOGOUT, ">> $logfile") or die "couldn't open $logfile\n";
open(ANNOT, ">> $annotfile") or die "couldn't open $annotfile\n";
print EXPLOGOUT "- $mday $mon $year ${hour}:${min}:${sec}\n".$explogout;
print ANNOT $explogout;
close ANNOT;
close EXPLOGOUT;

# clean tex auxiliary files 
system("rubber --clean $instrfile $stimtexfile $questmpfile") 
    || print "Could not automatically delete auxiliary files (tex) -\nrubber not installed?\n" ;
sdfsdf

__END__
