
"""
Execute One Run of the Feature Verification Experiment

author: Gerhard Kremer, 2010

usage:
python vpropverify.py -l <language> [options]

purpose:
Displays text pairs of words in sequence 
(one above, one below centre of screen);
records participants reaction times 
and stores configurational and PC-performance data
"""

# import modules
import os
import sys
import time
import logging

from optparse import OptionParser
from datetime import datetime
import random

import VisionEgg
from VisionEgg.Core import get_default_screen, Viewport, FixationSpot, swap_buffers, FrameTimer
from VisionEgg.Textures import FixationCross
from VisionEgg.MoreStimuli import FilledCircle
from VisionEgg.FlowControl import Presentation, FunctionController
from VisionEgg.Text import Text
import pygame

# Vission Egg config
# Turn off GUI window when calling create_default_screen()
VisionEgg.config.VISIONEGG_GUI_INIT = 0
VisionEgg.config.VISIONEGG_GUI_ON_ERROR = 0
VisionEgg.config.VISIONEGG_TKINTER_OK = 0
VisionEgg.config.VISIONEGG_MESSAGE_LEVEL = 3 
VisionEgg.config.VISIONEGG_MONITOR_REFRESH_HZ = 60.0
VisionEgg.config.VISIONEGG_FULLSCREEN = 1
VisionEgg.config.VISIONEGG_SCREEN_W = 1440 
VisionEgg.config.VISIONEGG_SCREEN_H = 900
VisionEgg.config.VISIONEGG_FRAMELESS_WINDOW = 1
VisionEgg.logger.setLevel(VisionEgg.logging.INFO)
VisionEgg.config.VISIONEGG_MAXPRIORITY = 1
VisionEgg.config.VISIONEGG_DOUBLE_BUFFER = 1

VisionEgg.watch_exceptions()
VisionEgg.start_default_logging()


# prepare stimulus set for the experiment
# (read/save data, set filename/subjNumber)
# create empty structure to save data to (mem. alloc.)
#######################################################################

# Initialisation
dtstart = datetime.now().strftime("%d. %b %Y %H:%M:%S")
stimuliset_split = False
trialdata = []
trialno = 0;

# Experiment Config
resolution =(VisionEgg.config.VISIONEGG_SCREEN_W,
             VisionEgg.config.VISIONEGG_SCREEN_H)
refreshrate = VisionEgg.config.VISIONEGG_MONITOR_REFRESH_HZ
fix1duration = (30, 'frames') #(1.0, 'seconds')
fix2duration = (10, 'frames') #(1.0, 'seconds')
fix2jitter = (60, 'frames') # 1000 milliseconds  
conduration = (30, 'frames') #(1.0, 'seconds')
conpropduration = (120, 'frames')
response_framescan_duration = 0.01496 #0.01666 #(real limit)
conpropfontsize = 70
conpropabovebelow = 105
laptoptype= 'D620'#'D630'  #laptoptype/-model/-no on which this is run


# Usage and Options
usagetext = '%prog [options]\n    ... option -h for help'
oparser = OptionParser(usage = usagetext)

oparser.add_option('-t', '--test', '--testrun',
                   action='store_true', default=False,
                   dest='testrun',
                   help='perform a testrun only, default:%default')

oparser.add_option('-l', '--lang', '--language',
                   dest='lang', nargs=1,
                   metavar='(DE|IT)',
                   help="experiment language is... (German or Italian)")
oparser.add_option('-d', '-D', '--DE', '--de',
                   action='store_const', const = 'DE', dest='lang')
oparser.add_option('-i', '-I', '--IT', '--it', 
                   action='store_const', const = 'IT', dest='lang')

(options, args) = oparser.parse_args()

if options.lang == None:
    oparser.error('You have to specify the language')
elif not (options.lang == 'DE' or options.lang == 'IT'):
    oparser.error('You have specified an unknown language tag')

# File Names
datafilename = 'subjdata'+options.lang+'.txt'
testrundatafilename = 'subjtestdata'+options.lang+'.txt'
if options.testrun == True: 
    dataoutputfilename = testrundatafilename
elif options.testrun == False:
    dataoutputfilename = datafilename
stimulitmpfilename = 'stimuli_tmp'+options.lang+'.txt'
stimulifilename = 'stimuli'+options.lang+'.txt'
testrunstimulifilename = 'testrun-stimuli'+options.lang+'.txt'
falsestimulitmpfilename = 'false-stimuli_tmp'+options.lang+'.txt'
falsestimulifilename = 'false-stimuli'+options.lang+'.txt'
expconfigfilename = 'expconfig'+options.lang+'.txt'

# SubjTag
# find subject tag (number) in datafile created in previous run
# or start with one if datafile not created already
if not os.path.isfile(datafilename):
    subjtag = 'subj'+str(1).zfill(3)
else:
    datafile = open(datafilename,'r')
    datalines = datafile.readlines()
    datafile.close()
    datalastline = datalines[len(datalines)-2]
    oldsubjtag = datalastline.split()[0]
    oldsubjtagnumber = oldsubjtag.lstrip('sbuj')
    nextsubjnumber = str(int(oldsubjtagnumber)+1).zfill(3)
    subjtag = 'subj'+nextsubjnumber


# Stimuli Preparation
######################################################################

# testrun: with special stimuli
if options.testrun == True: 
    stimulifile = open(testrunstimulifilename)
    stimulilines = stimulifile.readlines()
    stimulifile.close()
    concepts = [w[0] for w in [line.split() for line in stimulilines]]
    properties = [w[1] for w in [line.split() for line in stimulilines]]
    classes = [w[2] for w in [line.split() for line in stimulilines]]
    relations = [w[3] for w in [line.split() for line in stimulilines]]
    englwords = [w[4] for w in [line.split() for line in stimulilines]]
    vfvalues = [w[5] for w in [line.split() for line in stimulilines]]
    conpropairs = zip(concepts,properties,
                        classes, relations,
                        englwords,vfvalues)
else:
# read stimuli from temp file (if split in last session)
    if os.path.isfile(stimulitmpfilename):
        stimulifile = open(stimulitmpfilename, 'r')
        stimulilines = stimulifile.readlines()
        stimulifile.close()
        falsestimulifile = open(falsestimulitmpfilename, 'r')
        falsestimulilines = falsestimulifile.readlines()
        falsestimulifile.close()
    # there is no temp file from last session: read stimulifile
    #... and split into two sets
    else: 
        stimuliset_split = True
        stimulifile = open(stimulifilename, 'r')
        allstimulilines = stimulifile.readlines()
        stimulifile.close()
        random.seed()
        random.shuffle(allstimulilines)
        stimulitmplines = allstimulilines[:len(allstimulilines)/2]
        stimulilines = allstimulilines[len(allstimulilines)/2:]
        falsestimulifile = open(falsestimulifilename, 'r')
        allfalsestimulilines = falsestimulifile.readlines()
        falsestimulifile.close()
        random.seed()
        random.shuffle(allfalsestimulilines)
        falsestimulitmplines = allfalsestimulilines[:len(allfalsestimulilines)/2]
        falsestimulilines = allfalsestimulilines[len(allfalsestimulilines)/2:]
        
    # end if
    # pair stimuli and add stimuli w/ condition: false
    concepts = [w[0] for w in [line.split() for line in stimulilines]]
    properties = [w[1] for w in [line.split() for line in stimulilines]]
    classes = [w[2] for w in [line.split() for line in stimulilines]]
    relations = [w[3] for w in [line.split() for line in stimulilines]]
    englwords = [w[4] for w in [line.split() for line in stimulilines]]
    conpropairs = zip(concepts,properties,
                      classes, relations,
                      englwords,['valid']*len(concepts))
    falseconcepts = [w[0] for w in [line.split() for line in falsestimulilines]]
    falseproperties = [w[1] for w in [line.split() for line in falsestimulilines]]
    falseclasses = [w[2] for w in [line.split() for line in falsestimulilines]]
    falserelations = [w[3] for w in [line.split() for line in falsestimulilines]]
    falseenglwords = [w[4] for w in [line.split() for line in falsestimulilines]]

    falseconpropairs = zip(falseconcepts, falseproperties,
                            falseclasses, falserelations,
                            falseenglwords,['false']*len(falseconcepts))

    conpropairs = conpropairs + falseconpropairs
# end if

# randomise stimuli
#randstimulilines = numpy.random.permutation()
random.seed()
random.shuffle(conpropairs)
del(stimulilines,concepts,properties)

# add some filler items at the start (only in real experiment run)
# (hard-coded)
if options.testrun == False:
    # randomise fillers 
    if options.lang == 'DE':
        random.seed()
        starters = [
            ('Schuh','Ferse','clothing','part','shoe','starter'),
            ('Sofa','Schlafzimmer','furniture','location','sofa','starter'),
            ('Fahrrad','Sattel','vehicle','part','bicycle','starter'),
            ('Hemd','gemustert','clothing','quality','shirt','starter'),
            ('Ananas','braun','fruit','quality','ananas','starter')
            ]
        random.shuffle(starters)
        conpropairs = starters + conpropairs
    elif options.lang == 'IT':
        random.seed()
        starters = [
            ('scarpa','caviglia','clothing','part','shoe','starter'),
            ('divano','camera','furniture','location','sofa','starter'),
            ('bicicletta','sella','vehicle','part','bicycle','starter'),
            ('camicia','squadrato','clothing','quality','shirt','starter'),
            ('ananas','marrone','fruit','quality','ananas','starter')
            ]
        random.shuffle(starters)
        conpropairs = starters + conpropairs


# functions for controllers (key presses) and events
#################################################################

# ESC key for stopping experiment (used during Fixation point 2)
def keydown_trial(event):
    if event.key == pygame.locals.K_ESCAPE:
        sys.exit('\nProgram stopped by you during trial.\n')

# stop current stimulus presentation instantly when response given 
def exit_after_response(t):
    try:
        mouse_response.get_first_response_since_go()
    except AttributeError:
        return
    if len(mouse_response.get_first_response_since_go()) > 0:
        pconprop.parameters.go_duration = (0,'frames')

# at start: space to go on;
#           ESC to quit
def keydown_start(event):
    if event.key == pygame.locals.K_SPACE:
        quit()
    elif event.key == pygame.locals.K_ESCAPE:
        sys.exit('\nProgram stopped by you at start.\n')
        
def quit(dummy_arg=None):
    pstart.parameters.go_duration = (0,'frames')
    try:
        pend.parameters.go_duration = (0,'frames')
    except NameError:
        pass
    
# at end: space when done
def keydown_end(event):
    if event.key == pygame.locals.K_SPACE:
        quit()


# set up screen 
###############################################################

screen = get_default_screen()
screen.parameters.bgcolor = (0.0,0.0,0.0,0.0) # make black (RGBA)


# Welcome text
# [space] to go on
# [escape] to quit
###############################################################

# define text properties
startinfo = Text(text=subjtag+' ...   [ SPACE ]',
                 color=(1,1,1,1),
                 anchor='center',
                 position=(screen.size[0]/2,screen.size[1]-100),
                 font_size=50)

# set for presentation
vpstart = Viewport(screen=screen,
                          size=screen.size,
                          stimuli=[startinfo])

# prepare presentation
pstart = Presentation(go_duration=('forever', ),viewports=[vpstart])

pstart.parameters.handle_event_callbacks=[(pygame.locals.QUIT, quit),
                                          (pygame.locals.KEYDOWN, keydown_start)]

# execute presentation
pstart.go()
screen.clear();swap_buffers()
time.sleep(1)

######################################################################
# end of Welcome screen part



# set fixation circles (red, white)
# and their viewports
# (they don't change during experiment)
#######################################################################

fix1 = FilledCircle(
  anchor   = 'center',
  position = (screen.size[0]/2.0, screen.size[1]/2.0),
  radius   = 3.0,
  color    = (255, 0, 0) # Draw it in red
)

fix2 = FilledCircle(
  anchor   = 'center',
  position = (screen.size[0]/2.0, screen.size[1]/2.0),
  radius   = 3.0,
  color    = (255, 255, 255) # Draw it in white
)

vpfix1 = Viewport(screen=screen,
                  size=screen.size,
                  stimuli=[fix1])

vpfix2 = Viewport(screen=screen,
                  size=screen.size,
                  stimuli=[fix2])


# loop over all trials:
#######################################################################
for (conceptw,propw,conclass,proprel,englw,verificvalue) in conpropairs:

    #conceptw = '[concept]'
    #propw= '[property]'

    trialno += 1

# define text properties for concept word
    textc = Text(text=conceptw,
                color=(1.0,1.0,1.0),
                position=(screen.size[0]/2,
                          screen.size[1]/2+conpropabovebelow),
                font_size=conpropfontsize,
                anchor='center')

# define text properties of concept property word
    textp = Text(text=propw,
                color=(1.0,1.0,1.0),
                position=(screen.size[0]/2,
                          screen.size[1]/2-conpropabovebelow),
                font_size=conpropfontsize,
                anchor='center')

    vpcon = Viewport(screen=screen,
                     size=screen.size,
                     stimuli=[textc])

    vpconprop = Viewport(screen=screen,
                         size=screen.size,
                         stimuli=[textp])


    # presentations settings
    ##############################################

    pfix1 = Presentation(go_duration= fix1duration,
                         viewports=[vpfix1])

    # add jitter to ITI (in seconds/frames)
    if fix2duration[1] != fix2jitter[1]:
        sys.exit('\n Duration and jitter unit of fixation 2 differ!\n')
    elif fix2jitter[1] == 'seconds':
        random.seed()
        jit_duration = random.randint(0,fix2jitter[0]*1000)/1000
    elif fix2jitter[1] == 'frames':
        random.seed()
        jit_duration = random.randint(0,fix2jitter[0]/10)*10

    pfix2 = Presentation(go_duration=((fix2duration[0]+jit_duration),
                                      fix2duration[1]),
                         viewports=[vpfix2])
    pfix2.parameters.handle_event_callbacks=[(pygame.locals.QUIT, quit),
                                             (pygame.locals.KEYDOWN, keydown_trial)]
    pcon = Presentation(go_duration=conduration,
                        viewports=[vpcon])
    pconprop = Presentation(go_duration= conpropduration,
                            viewports=[vpconprop])

    pempty = Presentation(go_duration = (30,'frames'), viewports=[])

    
    # presentations 
    # of fix point 1, concept word, empty screen
    # (start logger first)
    logger = logging.getLogger('VisionEgg.FlowControl')
    logger.setLevel(logging.CRITICAL)

    pfix1.go()
    pcon.go()
    screen.clear()
    swap_buffers()
    screen.clear()
    pempty.go()
    logging.debug('\n--'+subjtag+'\n')

    # present property word and start polling for mouse response
    # during presentation; 
    # stop presetation instantly when response given
    if conpropduration[1] != 'frames':
        sys.exit('Only frame durations can be used in this experiment')
    frame_timer = FrameTimer() # start frame counter/timer
    mouse_response = []
    time_mouse_response = []
    buttons = ()
    frames_since_start = 0
    time_response_start = time.time()
    while frames_since_start < conpropduration[0]:
        screen.clear()
        # present property word now
        vpconprop.draw()
        swap_buffers() # VisionEgg.Core
        frame_timer.tick()
        frames_since_start += 1
        time_frame_start = time.time()
        time_response_scan = time.time()
        while ( (not 1 in buttons) 
                and (time_response_scan - time_frame_start) 
                < response_framescan_duration):
            pygame.event.get()
            buttons = pygame.mouse.get_pressed()
            time_response_scan = time.time()
        # end of response loop
        if 1 in buttons:
            break
    # end of presentation loop
    # check what was pressed
    pressed = [c for c, v in enumerate(buttons) if v]
    if len(pressed) > 1:
        mouse_response.append('MB_MULTIPLE')
    elif pressed == [0]:
        mouse_response.append('MB_LEFT')
    elif pressed == [2]:
        mouse_response.append('MB_RIGHT')
    elif pressed == [1]:
        mouse_response.append('MB_MIDDLE')
    elif pressed == []:
        mouse_response.append('MB_NONE')
    if 1 in buttons:
        time_mouse_response = time_response_scan - time_response_start
    else:
        time_mouse_response = 'NoRT'
    frame_timer.log_histogram()


#######################################################################

    # present inter-trial fixpoint
    pfix2.go()

    singletrialdata=' '.join([subjtag, 
                              laptoptype, 
                              'trial'+str(trialno).zfill(3),
                              conceptw, propw, verificvalue,
                              mouse_response[0], str(time_mouse_response),
                              conclass, proprel, englw
                              ])
    trialdata.append(singletrialdata)

########################################################################
#
# end of presentation loop



# save data, clean up
#######################################################################

# write subject data file
dtend = datetime.now().strftime('%d. %b %Y %H:%M:%S')
datafile=open(dataoutputfilename,'a')
datafile.write('\n'.join(trialdata)+'\n--... recorded: '+dtend+'\n')
datafile.close()
print '\n'.join(trialdata)

# write expconfigfile
expconfigfile = open(expconfigfilename,'a')
expconfigfile.write('\n'.join([subjtag,
                              'Resolution: '
                              + ' '.join([str(e) for e in resolution]),
                              'Refresh rate: '
                              + str(refreshrate),
                              'Font size: '
                              + str(conpropfontsize),
                              'Vertical pixel distance of center of text'
                              + 'from screen center: '
                              + str(conpropabovebelow),
                              'Fixation duration before stimulus: '
                              + ' '.join([str(e) for e in fix1duration]),
                              'Concept presentation duration (SOA): '
                              + ' '.join([str(e) for e in conduration]),
                              'Concept-property pair presentation duration'
                              + ' (upper time limit): '
                              + ' '.join([str(e) for e in conpropduration]),
                              'Fixation duration after stimulus: '
                              + ' '.join([str(e) for e in fix2duration])
                              + ' plus jitter of '
                              +  ' '.join([str(e) for e in fix2jitter]),
                              ])
                         + '\n--... recorded: '+ dtstart+ ' - '+ dtend+ '\n')
expconfigfile.close()

# remove semi-stimuli-subset file if necessary 
# (i.e. if used in this experiment session)
# or write semi-stimuli-subset file, if just created
if stimuliset_split == False and options.testrun == False:
    os.remove(stimulitmpfilename)
    os.remove(falsestimulitmpfilename)
elif stimuliset_split == True and options.testrun == False:
    stimulitmpdata = '\n'.join([l.rstrip() for l in stimulitmplines])
    stimulitmpfile = open(stimulitmpfilename,'w')
    stimulitmpfile.write(stimulitmpdata+'\n')
    stimulitmpfile.close()
    falsestimulitmpdata = '\n'.join([l.rstrip() for l in falsestimulitmplines])
    falsestimulitmpfile = open(falsestimulitmpfilename,'w')
    falsestimulitmpfile.write(falsestimulitmpdata+'\n')
    falsestimulitmpfile.close()
elif options.testrun == True:
    pass


# Thank-you frame
#################################################################

endinfo = Text(text='Grazie !      '+subjtag+'      Danke !',
               color=(1,1,1,1),
               anchor='center',
               position=(screen.size[0]/2,screen.size[1]/2),
               font_size=50)

vpend = Viewport(screen=screen,
                 size=screen.size,
                 stimuli=[endinfo])

pend = Presentation(go_duration=('forever', ),viewports=[vpend])

pend.parameters.handle_event_callbacks=[(pygame.locals.QUIT, quit),
                                        (pygame.locals.KEYDOWN, keydown_end)]

pend.go()


print '\n\n '+subjtag+' data has been recorded successfully...\n\n'



######################################################################
######################################################################
